{% extends "base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Error 404" %}{% endblock %}

{% block content %}
<h1>{% trans "Error 404" %}</h1>
<p>{% trans "We are sorry, but this page was not found." %}</p>
{% endblock %}
