.fw-contents h1 {
    font-size: 2em;
    margin: .67em 0;
    font-weight: 700;
}

.fw-contents h2 {
    font-size: 1.5em;
    margin: .75em 0;
    font-weight: 700;
}

.fw-contents h3 {
    font-size: 1.17em;
    margin: .83em 0;
    font-weight: 700;
}

.fw-contents h4 {
    margin: 1.12em 0;
    font-weight: 700;
}

.fw-contents h5 {
    font-size: .83em;
    margin: 1.5em 0;
    font-weight: 700;
}

.fw-contents h6 {
    font-size: .75em;
    margin: 1.67em 0;
    font-weight: 700;
}

.fw-contents ul {
    list-style-type: disc;
    list-style-position: inside;
}

.fw-contents ol {
    list-style-type: decimal;
    list-style-position: inside;
}
