table td {
    vertical-align: top;
}

.label {
    padding: 5px 0 5px 0;
}

.from-container,
.to-column,
.trash {
    border: 1px solid var(--cs-white-border);
    background-color: var(--cs-white-background);
}

.to-container {
    min-height: 500px;
}

.to-column .smooth-dnd-draggable-wrapper {
    overflow: visible !important;
}

.to-column {
    width: 650px;
}

.to-column .article-part table td {
    border: 1px solid var(--cs-white-border);
}

.from-container {
    width: 150px;
}

.trash {
    background-color: var(--cs-dark-background);
    width: 100px;
}

.doc-part {
    background-color: var(--part-neutral-background-color);
    border-bottom: 3px dotted var(--part-neutral-border-color);
    padding: 4px;
}

.doc-part[data-type='heading_part'] {
    background-color: var(--part-heading-background-color);
}

.doc-part[data-type='contributors_part'] {
    background-color: var(--part-contributors-background-color);
}

.doc-part[data-type='richtext_part'] {
    background-color: var(--part-richtext-background-color);
}

.doc-part[data-type='tags_part'] {
    background-color: var(--part-tags-background-color);
}

.doc-part[data-type='table_part'] {
    background-color: var(--part-table-background-color);
}

.doc-part[data-type='table_of_contents'] {
    background-color: var(--part-toc-background-color);
}

.from-container .doc-part,
.trash .doc-part {
    height: 50px;
}

.from-container .attrs,
.from-container .object-tools,
.trash .attrs,
.trash .object-tools,
.from-container .doc-part-header .label,
.attrs.hidden {
    display: none;
}

.attrs label {
    display: inline-block;
    width: initial;
    float: initial;
}

.doc-part-header {
    cursor: move; /* fallback if grab cursor is unsupported */
    cursor: grab;
    height: 60px;
}

.from-container .doc-part-header {
    height: 100%;
}

.object-tools.right {
    float: right;
}

.ProseMirror {
    background-color: var(--cs-white-background);
    color: var(--cs-white-text);
}

.ProseMirror-menubar {
    background-color: inherit;
    position: inherit;
    min-height: 21px !important;
}

.article-table th {
    background-color: var(--cs-6-background);
    color: var(--cs-6-text);
}

.ProseMirror td.selectedCell {
    background-color: var(--cs-2-background);
    caret-color: transparent;
}

.error-element {
    border: 3px solid var(--error-background-color);
}

#template-editor .initial em,
#template-editor .instructions em {
    font-style: italic;
}

#template-editor .initial strong,
#template-editor .instructions strong {
    font-weight: bold;
}

#template-editor .initial code,
#template-editor .instructions code {
    font-family: monospace;
}

#template-editor .initial h1,
#template-editor .instructions h1 {
    font-size: 28px;
}

#template-editor .initial h2,
#template-editor .instructions h2 {
    font-size: 24px;
}

#template-editor .initial h3,
#template-editor .instructions h3 {
    font-size: 20px;
}

#template-editor .initial h4,
#template-editor .instructions h4 {
    font-size: 16px;
}

#template-editor .initial h5,
#template-editor .instructions h5 {
    font-size: 14px;
}

#template-editor .initial h6,
#template-editor .instructions h6 {
    font-size: 12px;
}

#template-editor .initial ul,
#template-editor .instructions ul {
    list-style-type: disc;
}

#template-editor .initial ol,
#template-editor .instructions ol {
    list-style-type: decimal;
}

.object-tools span.link {
    display: block;
    cursor: pointer;
    border-radius: 15px;
    background-color: var(--cs-2-background);
    color: var(--cs-2-text);
    padding: 3px 12px;
}
