"""
Application that contains main app settings
(Hosts, middleware, templates, installed apps)
Urls to route between applications,
statics that are shared among different django applications
Also contains helper for starting tornado server and management commands
"""
default_app_config = 'base.apps.BaseConfig'
