export {ContributorDialog} from "./contributor"
export {CitationDialog} from "./citation"
export {FigureDialog} from "./figure"
export {MathDialog} from "./math"
export {LanguageDialog} from "./language"
export {LinkDialog} from "./link"
export {TableDialog, TableResizeDialog} from "./table"
export {RevisionDialog} from "./revision"
